Package: hesim
Type: Package
Title: Health Economic Simulation Modeling and Decision Analysis
Version: 0.5.8
Authors@R: c(
    person("Devin", "Incerti", , "devin.incerti@gmail.com", role = c("aut", "cre")),
    person("Jeroen P.", "Jansen", role = "aut"),
    person("Mark", "Clements", role = "aut"),
    person(given = "R Core Team", role = "ctb",
    comment="hesim uses some slightly modified C functions from base R")
    )
Description: A modular and computationally efficient R package for  
  parameterizing, simulating, and analyzing health economic simulation 
  models. The package supports cohort discrete time state transition models 
  (Briggs et al. 1998) <doi:10.2165/00019053-199813040-00003>,
  N-state partitioned survival models (Glasziou et al. 1990)
  <doi:10.1002/sim.4780091106>, and individual-level continuous 
  time state transition models (Siebert et al. 2012) <doi:10.1016/j.jval.2012.06.014>,
  encompassing both Markov (time-homogeneous and time-inhomogeneous) and 
  semi-Markov processes. Decision uncertainty from a cost-effectiveness analysis is 
  quantified with standard graphical and tabular summaries of a probabilistic 
  sensitivity analysis (Claxton et al. 2005, Barton et al. 2008) <doi:10.1002/hec.985>, 
  <doi:10.1111/j.1524-4733.2008.00358.x>. Use of C++ and data.table
  make individual-patient simulation, probabilistic sensitivity analysis, 
  and incorporation of patient heterogeneity fast.
URL: https://hesim-dev.github.io/hesim/,
        https://github.com/hesim-dev/hesim
BugReports: https://github.com/hesim-dev/hesim/issues
License: GPL-3
LazyData: true
LinkingTo: Rcpp, RcppArmadillo
Depends: R (>= 3.5.0)
Imports: data.table, flexsurv, ggplot2, MASS, msm, Rcpp (>= 0.12.16),
        R6, stats, survival
Suggests: covr, kableExtra, knitr, magrittr, mstate, nnet, numDeriv,
        pracma, rmarkdown, scales, testthat, truncnorm
VignetteBuilder: knitr
RoxygenNote: 7.3.3
NeedsCompilation: yes
Packaged: 2026-01-12 23:35:33 UTC; devin
Author: Devin Incerti [aut, cre],
  Jeroen P. Jansen [aut],
  Mark Clements [aut],
  R Core Team [ctb] (hesim uses some slightly modified C functions from
    base R)
Maintainer: Devin Incerti <devin.incerti@gmail.com>
Repository: CRAN
Date/Publication: 2026-01-13 00:00:02 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-26 18:57:27 UTC; unix
Archs: hesim.so.dSYM
