# Generated by roxygen2: do not edit by hand

S3method("[",expanded_hesim_data)
S3method(as.data.table,input_mats)
S3method(as.data.table,tparams_transprobs)
S3method(as.list,eval_rng)
S3method(autoplot,stateprobs)
S3method(autoplot,survival)
S3method(c,eval_rng)
S3method(cbind,expanded_hesim_data)
S3method(cea,ce)
S3method(cea,default)
S3method(cea_pw,ce)
S3method(cea_pw,default)
S3method(create_CohortDtstm,model_def)
S3method(create_CohortDtstmTrans,msm)
S3method(create_CohortDtstmTrans,multinom_list)
S3method(create_CohortDtstmTrans,params_mlogit_list)
S3method(create_IndivCtstmTrans,flexsurvreg)
S3method(create_IndivCtstmTrans,flexsurvreg_list)
S3method(create_IndivCtstmTrans,params_surv)
S3method(create_IndivCtstmTrans,params_surv_list)
S3method(create_PsmCurves,flexsurvreg_list)
S3method(create_PsmCurves,params_surv_list)
S3method(create_StateVals,eval_model)
S3method(create_StateVals,lm)
S3method(create_StateVals,stateval_tbl)
S3method(create_input_mats,flexsurvreg)
S3method(create_input_mats,flexsurvreg_list)
S3method(create_input_mats,formula_list)
S3method(create_input_mats,lm)
S3method(create_input_mats,multinom)
S3method(create_input_mats,multinom_list)
S3method(create_input_mats,params_lm)
S3method(create_input_mats,params_mlogit_list)
S3method(create_input_mats,params_surv)
S3method(create_input_mats,params_surv_list)
S3method(create_input_mats,partsurvfit)
S3method(create_params,flexsurvreg)
S3method(create_params,flexsurvreg_list)
S3method(create_params,lm)
S3method(create_params,multinom)
S3method(create_params,multinom_list)
S3method(create_params,partsurvfit)
S3method(expand,hesim_data)
S3method(format,icer)
S3method(format,summary.ce)
S3method(plot_ceac,cea)
S3method(plot_ceac,cea_pw)
S3method(plot_ceac,default)
S3method(print,eval_rng)
S3method(print,input_mats)
S3method(print,params_lm)
S3method(print,params_mlogit)
S3method(print,params_mlogit_list)
S3method(print,params_surv)
S3method(print,params_surv_list)
S3method(print,tparams_mean)
S3method(print,tparams_transprobs)
S3method(qmatrix,data.frame)
S3method(qmatrix,data.table)
S3method(qmatrix,matrix)
S3method(qmatrix,msm)
S3method(sim_ev,"NULL")
S3method(sim_ev,stateprobs)
S3method(sim_stateprobs,survival)
S3method(summary,ce)
S3method(summary,eval_rng)
S3method(summary,params_lm)
S3method(summary,params_mlogit)
S3method(summary,params_mlogit_list)
S3method(summary,params_surv)
S3method(summary,params_surv_list)
S3method(summary,tparams_mean)
S3method(summary,tparams_transprobs)
S3method(summary,tpmatrix)
S3method(tparams_transprobs,array)
S3method(tparams_transprobs,data.frame)
S3method(tparams_transprobs,data.table)
S3method(tparams_transprobs,eval_model)
S3method(tparams_transprobs,tpmatrix)
export(CohortDtstm)
export(CohortDtstmTrans)
export(CtstmTrans)
export(HweibullNMA)
export(IndivCtstm)
export(IndivCtstmTrans)
export(Psm)
export(PsmCurves)
export(StateVals)
export(apply_rr)
export(as.data.table)
export(as_array3)
export(as_pfs_os)
export(as_tbl2)
export(autoplot)
export(cea)
export(cea_pw)
export(create_CohortDtstm)
export(create_CohortDtstmTrans)
export(create_IndivCtstmTrans)
export(create_PsmCurves)
export(create_StateVals)
export(create_input_mats)
export(create_lines_dt)
export(create_params)
export(create_trans_dt)
export(define_model)
export(define_rng)
export(define_tparams)
export(dweibullNMA)
export(eval_model)
export(eval_rng)
export(expand)
export(expmat)
export(fast_rgengamma)
export(flexsurvreg_list)
export(formula_list)
export(get_labels)
export(hesim_data)
export(hesim_survdists)
export(hweibullNMA)
export(icea)
export(icea_pw)
export(icer)
export(icer_tbl)
export(id_attributes)
export(incr_effect)
export(input_mats)
export(lm_list)
export(mean_weibullNMA)
export(mom_beta)
export(mom_gamma)
export(multinom_list)
export(params_lm)
export(params_mlogit)
export(params_mlogit_list)
export(params_surv)
export(params_surv_list)
export(partsurvfit)
export(plot_ceac)
export(plot_ceaf)
export(plot_ceplane)
export(plot_evpi)
export(pweibullNMA)
export(qmatrix)
export(qweibullNMA)
export(rcat)
export(rdirichlet_mat)
export(rmst_weibullNMA)
export(rpwexp)
export(rweibullNMA)
export(set_labels)
export(sim_costs)
export(sim_ev)
export(sim_qalys)
export(sim_stateprobs)
export(stateval_tbl)
export(surv_quantile)
export(survival)
export(time_intervals)
export(tparams_mean)
export(tparams_transprobs)
export(tpmatrix)
export(tpmatrix_id)
export(tpmatrix_names)
import(data.table)
importFrom(R6,R6Class)
importFrom(Rcpp,evalCpp)
importFrom(data.table,as.data.table)
importFrom(ggplot2,.data)
importFrom(ggplot2,autoplot)
useDynLib(hesim)
